@@ -1,4 +1,3 @@
-# coding: utf8
 from __future__ import print_function
 import logging
 import sys
@@ -11,11 +10,11 @@ import wsgiref.validate
 from wsgiref.simple_server import make_server
 try:
     from http.cookies import BaseCookie
-    from urllib.parse import quote, unquote, urlparse, urlunparse
+    from urllib.parse import unquote, urlparse, urlunparse
 except ImportError:
     from Cookie import BaseCookie  # NOQA
     from urlparse import urlparse, urlunparse  # NOQA
-    from urllib import quote, unquote  # NOQA
+    from urllib import unquote  # NOQA
 
 from functools import wraps
 from itertools import chain
@@ -26,12 +25,10 @@ from lxml.html import fromstring, tostring
 from lxml.cssselect import CSSSelector, SelectorSyntaxError
 from lxml.etree import XPath, XPathError
 
-from pesto.testing import MockResponse
-from pesto.request import Request
-from pesto.wsgiutils import uri_join, make_query
-from pesto.httputils import parse_header
-
-__version__ = '7.3.7'
+from fresco import Request, Response
+from fresco.util.urls import url_join, make_query
+from fresco.util.http import parse_header
+from fresco.util.wsgi import unicode_to_environ
 
 #: Registry for xpath multimethods
 xpath_registry = {}
@@ -78,7 +75,7 @@ class XPathMultiMethod(object):
         for xpath, func in self.endpoints:
             if el in xpath(el):
                 return func(*args, **kwargs)
-        raise NotImplementedError("Function %s not implemented for element %r"
+        raise NotImplementedError("Function %s not implemented for element %r"\
                                   % (self.__name__, el,))
 
     def register(self, xpath, func):
@@ -94,9 +91,12 @@ class XPathMultiMethod(object):
         # Note that ".. comment block" is required to fool rst/sphinx into
         # correctly parsing the indented paragraphs when there is only one
         # registered endpoint.
-        doc = 'For elements matching ``%s``:n%s\n\n.. comment block\n\n' % (
+        doc = 'For elements matching ``%s``:'\
+              '\n%s\n\n.. comment block\n\n' % (
                     xpath,
-                  '\n'.join('    %s' % line for line in func_doc.split('\n')))
+                    '\n'.join('    %s' % line
+                              for line in func_doc.split('\n'))
+        )
         self.__doc__ += doc
         self.__name__ = func.__name__
 
@@ -129,7 +129,7 @@ class ElementWrapper(object):
     normally created through the ``find``/``css`` methods of ``TestAgent``
     instance::
 
-        >>> from pesto.response import Response
+        >>> from fresco import Response
         >>> myapp = Response(['<html><body><a href="/">link 1</a>'\
         ...                   '<a href="/">link 2</a></body></html>'])
         >>> agent = TestAgent(myapp).get('/')
@@ -387,7 +387,7 @@ class ElementWrapper(object):
         group)
         """
         for el in self.el.xpath(
-            "./ancestor-or-self::form[1]"
+            "./ancestor-or-self::form[1]"\
             "//input[@type='radio' and @name=$name]",
             name=self.el.attrib.get('name', '')
         ):
@@ -429,8 +429,9 @@ class ElementWrapper(object):
         previously selected option.
         """
         if 'multiple' not in \
-                self.el.xpath('./ancestor-or-self::select[1]')[0].attrib:
-            for el in self.el.xpath("./ancestor-or-self::select[1]//option"):
+           self.el.xpath('./ancestor-or-self::select[1]')[0].attrib:
+            for el in \
+            self.el.xpath("./ancestor-or-self::select[1]//option"):
                 if 'selected' in el.attrib:
                     del el.attrib['selected']
 
@@ -470,8 +471,8 @@ class ElementWrapper(object):
         method = self.el.attrib.get('method', 'GET').upper()
         data = self.submit_data(button)
         path = url_join_same_server(
-            self.agent.request.request_uri,
-            self.el.attrib.get('action', self.agent.request.request_path)
+            self.agent.request.url,
+            self.el.attrib.get('action', self.agent.request.path)
         )
         return {
             ('GET', None): self.agent.get,
@@ -550,18 +551,14 @@ class ElementWrapper(object):
             self(selector).fill(value)
 
         for name, value in kwargs.items():
-            path = ".//*[(local-name() = 'input' "\
-                   "or local-name() = 'textarea' "\
-                   "or local-name() = 'select') "\
-                   "and (@name=$name or @id=$name)]"
-            element = self.find(path, name=name)
-            if len(element) == 0:
-                fields = ', '.join(e.name
-                                   for e in self('css, textarea, select'))
-                raise IndexError(
-                    "Couldn't find a form element named {0!r}. "
-                    "Valid names are {1}".format(name, fields))
-            element.fill(value)
+            path = """
+            .//*[
+                (local-name() = 'input'\
+                 or local-name() = 'textarea'\
+                 or local-name() = 'select')
+                and (@name=$name or @id=$name)
+            ]"""
+            self.find(path, name=name).fill(value)
 
         return self
 
@@ -593,8 +590,8 @@ class ElementWrapper(object):
                 elif 'checked' in el.attrib:
                     del el.attrib['checked']
             if found != values:
-                raise AssertionError("Values %r not present"
-                                     " in checkbox group %r" %
+                raise AssertionError("Values %r not present"\
+                                     " in checkbox group %r" %\
                                      (values - found,
                                       self.el.attrib.get('name')))
 
@@ -619,8 +616,8 @@ class ElementWrapper(object):
             elif 'checked' in el.attrib:
                 del el.attrib['checked']
         if value is not None and not found:
-            raise AssertionError("Value %r not present"
-                                 " in radio button group %r" %
+            raise AssertionError("Value %r not present"\
+                                 " in radio button group %r" %\
                                  (value, self.el.attrib.get('name')))
         return self
 
@@ -709,7 +706,7 @@ class ElementWrapper(object):
             elif 'selected' in opt.attrib:
                 del opt.attrib['selected']
         if not found and value is not None:
-            raise AssertionError("Value %r not present in select %r" %
+            raise AssertionError("Value %r not present in select %r" %\
                                  (value, self.el.attrib.get('name')))
         return self
 
@@ -739,7 +736,7 @@ class ElementWrapper(object):
 
         Example::
 
-            >>> from pesto.response import Response
+            >>> from fresco import Response
             >>> myapp = Response(['<p>the <span>foo</span> is'\
             ...                   ' completely <strong>b</strong>azzed</p>'])
             >>> agent = TestAgent(myapp).get('/')
@@ -770,7 +767,7 @@ class ResultWrapper(list):
     turn wrap an lxml element and are normally created through the find/findcss
     methods of ``TestAgent``::
 
-        >>> from pesto.response import Response
+        >>> from fresco import Response
         >>> myapp = Response(['<html><p>item 1</p><p>item 2</p></html>'])
         >>> agent = TestAgent(myapp).get('/')
         >>> resultwrapper = agent.find('//p')
@@ -853,7 +850,7 @@ class TestAgent(object):
           requests for the WSGI application and return a new ``TestAgent``
           object
 
-        - ``request``, ``response`` - the `Pesto <http://pesto.redgecko.org/>`_
+        - ``request``, ``response`` - the `Fresco <http://fresco.redgecko.org/>`_
           request and response objects associated with the last WSGI request.
 
         - ``body`` - the body response as a bytes object
@@ -871,7 +868,7 @@ class TestAgent(object):
            ``ResultWrapper`` object.
     """
 
-    response_class = MockResponse
+    response_class = Response
 
     default_charset = 'UTF-8'
 
@@ -896,9 +893,7 @@ class TestAgent(object):
 
     def __init__(self, app, request=None, response=None, cookies=None,
                  history=None, validate_wsgi=True, host='127.0.0.1',
-                 port='8080', loglevel=None, logger=None,
-                 environ_defaults=None,
-                 close_response=True):
+                 port='8080', loglevel=None, logger=None, close_response=True):
         """
         Initialize the ``TestAgent`` object.
 
@@ -920,12 +915,11 @@ class TestAgent(object):
 
         The following parameters are intended for internal use only:
 
-        :param environ_defaults: A list of default WSGI environ values
         :param history: The history list
         :param cookie: The cookie store to use for the TestAgent.
-        :param request: The ``pesto.request.Request`` object associated with
+        :param request: The ``fresco.request.Request`` object associated with
                         the current request
-        :param response: The ``pesto.request.Response`` object associated with
+        :param response: The ``fresco.request.Response`` object associated with
                          the current request
         """
 
@@ -978,8 +972,7 @@ class TestAgent(object):
         # Stores file upload field values in forms
         self.file_uploads = {}
 
-        self.environ_defaults = (environ_defaults or self.environ_defaults)\
-                                    .copy()
+        self.environ_defaults = self.__class__.environ_defaults.copy()
         self.environ_defaults.update({'SERVER_NAME': host,
                                       'SERVER_PORT': str(port)})
 
@@ -1001,8 +994,8 @@ class TestAgent(object):
 
         if '?' in PATH_INFO:
             if 'QUERY_STRING' in kwargs:
-                raise AssertionError("QUERY_STRING specified both in "
-                                     "PATH_INFO and as argument to "
+                raise AssertionError("QUERY_STRING specified both in "\
+                                     "PATH_INFO and as argument to "\
                                      "make_environ")
             PATH_INFO, querystring = PATH_INFO.split('?', 1)
             kwargs['QUERY_STRING'] = unicode_to_environ(querystring)
@@ -1013,31 +1006,17 @@ class TestAgent(object):
         assert not re.search(r'%(?![A-F0-9]{2})', PATH_INFO), \
             "Path info not URL encoded"
 
-        # Unquote requires a string argument
-        if isinstance(PATH_INFO, bytes):
-            PATH_INFO = PATH_INFO.decode('ascii')
-        if isinstance(SCRIPT_NAME, bytes):
-            SCRIPT_NAME = SCRIPT_NAME.decode('ascii')
-
-        # 'caf%C3%A9' -> 'caf\xc3\xa9'
         PATH_INFO = unquote(PATH_INFO)
         SCRIPT_NAME = unquote(SCRIPT_NAME)
-
-        # Python 3 unquotes to unicode correctly but Python 2 gets it wrong
-        if not PY3:
-            # 'caf\xc3\xa0' -> b'caf\xc3\xa9' -> 'café'
-            PATH_INFO = PATH_INFO.encode('latin1').decode(self.charset)
-            SCRIPT_NAME = SCRIPT_NAME.encode('latin1').decode(self.charset)
-
         PATH_INFO = unicode_to_environ(PATH_INFO)
         SCRIPT_NAME = unicode_to_environ(SCRIPT_NAME)
 
         environ = self.environ_defaults.copy()
         environ.update(kwargs)
         for key, value in kwargs.items():
+            environ[key.replace('wsgi_', 'wsgi.')] = value
             if '.' not in key:
                 value = unicode_to_environ(value)
-            environ[key.replace('wsgi_', 'wsgi.')] = value
 
         if isinstance(wsgi_input, bytes):
             wsgi_input = BytesIO(wsgi_input)
@@ -1059,8 +1038,7 @@ class TestAgent(object):
 
         return environ
 
-    def _wsgi_request(self, environ, follow=True, history=False,
-                      check_status=True):
+    def _request(self, environ, follow=True, history=False, check_status=True):
         """
         Low level entry point for making requests to the WSGI application.
 
@@ -1091,7 +1069,7 @@ class TestAgent(object):
 
         if self.options['logger']:
             self.options['logger'].info("%s %s", environ['REQUEST_METHOD'],
-                                        Request(environ).request_uri)
+                                        Request(environ).url)
             if environ['HTTP_COOKIE']:
                 self.options['logger'].debug("Cookie: %s",
                                              environ['HTTP_COOKIE'])
@@ -1106,9 +1084,8 @@ class TestAgent(object):
             self.app,
             Request(environ),
             response,
-            self.cookies.copy(),
+            self.cookies,
             history,
-            environ_defaults=self.environ_defaults,
             **self.options
         )
         if self.options['logger']:
@@ -1127,113 +1104,135 @@ class TestAgent(object):
             return agent.follow_all()
         return agent
 
-    def _request(self, PATH_INFO='/', data=None, charset='UTF-8', follow=True,
-                 history=True, check_status=True,
-                 content_type='application/x-www-form-urlencoded',
-                 method='POST',
-                 **kwargs):
+    def get(self, PATH_INFO='/', data=None, charset='UTF-8', follow=True,
+            history=True, check_status=True, **kwargs):
         """
-        Make a POST request to the application and return the response.
+        Make a GET request to the application and return the response.
 
-        :param PATH_INFO: The path to request from the application. This must
-                          be URL encoded.
+        PATH_INFO
+            The path to request from the application. This must be URL encoded.
+            As a convenience, query string data can be appended to this.
+            Example value: ``agent.get('/my%20script?param=1')``
 
-        :param data: POST data to be sent to the application. Can be a byte
-                     string of the raw post data, a dict or a list of ``(name,
-                     value)`` tuples.
+        data
+            Query string data to be appended to the request, must be either a
+            dict or list of ``(name, value)`` tuples.
 
-        :param charset: Encoding used for any unicode values encountered in
-                        ``data``
+        charset
+            Encoding used for any unicode values encountered in ``data``
 
-        :param content_type: The content type header for the posted data. The
-                             default is good for testing form submissions, if
-                             you want to test an API you may need to change
-                             this to something else, eg 'application/json'
+        follow
+            If false, redirect responses will not be followed
 
-        :param follow: If false, redirect responses will not be followed
+        history
+            If true, a new entry will be added to the history attribute for the
+            resulting TestAgent object
 
-        :param history: If true, a new entry will be added to the history
-                        attribute for the resulting TestAgent object
+        check_status
+            If true, any non success HTTP status code will result in an
+            AssertionError being raised
 
-        :param check_status: If true, any non success HTTP status code will
-                             result in an AssertionError being raised
         """
+        if data is not None:
+            kwargs.setdefault('QUERY_STRING', make_query(data,
+                                                         charset=charset))
 
         if self.request:
-            PATH_INFO = url_join_same_server(self.request.request_uri,
+            PATH_INFO = url_join_same_server(self.request.url,
                                              PATH_INFO)
-        else:
-            baseurl = '{0}://{1}:{2}'.format(
-                            self.environ_defaults['wsgi.url_scheme'],
-                            self.options['host'],
-                            self.options['port'])
-            PATH_INFO = url_join_same_server(baseurl, PATH_INFO)
 
-        if data is None:
-            envargs = kwargs
-        else:
-            if not isinstance(data, bytes):
-                data = make_query(data, charset=charset).encode('ASCII')
-            envargs = {'CONTENT_TYPE': content_type,
-                       'CONTENT_LENGTH': str(len(data))}
-            envargs.update(kwargs)
-
-        wsgi_input = BytesIO(data)
-        wsgi_input.seek(0)
-
-        return self._wsgi_request(
+        return self._request(
             self.make_environ(
-                method,
+                'GET',
                 PATH_INFO,
-                wsgi_input=wsgi_input,
-                **envargs
+                **kwargs
             ),
             follow,
             history,
             check_status,
         )
 
-    def get(self, PATH_INFO='/', data=None, charset='UTF-8', *args, **kwargs):
-        kwargs.setdefault('method', 'GET')
-        if data is not None:
-            kwargs.setdefault('QUERY_STRING', make_query(data,
-                                                         charset=charset))
-            data = None
-        return self._request(PATH_INFO, data, charset, *args, **kwargs)
+    def post(self, PATH_INFO='/', data=None, charset='UTF-8', follow=True,
+             history=True, check_status=True, **kwargs):
+        """
+        Make a POST request to the application and return the response.
 
-    def head(self, *args, **kwargs):
-        return self.get(method='HEAD', *args, **kwargs)
+        PATH_INFO
+            The path to request from the application. This must be URL encoded.
 
-    def post(self, *args, **kwargs):
-        return self._request(method='POST', *args, **kwargs)
+        data
+            POST data to be sent to the application, must be either a
+            dict or list of ``(name, value)`` tuples.
 
-    def delete(self, *args, **kwargs):
-        return self._request(method='DELETE', *args, **kwargs)
+        charset
+            Encoding used for any unicode values encountered in ``data``
 
-    def put(self, *args, **kwargs):
-        return self._request(method='PUT', *args, **kwargs)
+        follow
+            If false, redirect responses will not be followed
 
-    def patch(self, *args, **kwargs):
-        return self._request(method='PATCH', *args, **kwargs)
+        history
+            If true, a new entry will be added to the history attribute for the
+            resulting TestAgent object
 
-    def post_multipart(self, PATH_INFO='/', data=None, charset='UTF-8',
-                       files=None, *args, **kwargs):
+        check_status
+            If true, any non success HTTP status code will result in an
+            AssertionError being raised
+
+
+        """
+        if data is None:
+            data = []
+
+        if self.request:
+            PATH_INFO = url_join_same_server(self.request.url,
+                                             PATH_INFO)
+
+        data = make_query(data, charset=charset).encode('ASCII')
+        wsgi_input = BytesIO(data)
+        wsgi_input.seek(0)
+
+        return self._request(
+            self.make_environ(
+                'POST',
+                PATH_INFO,
+                CONTENT_TYPE="application/x-www-form-urlencoded",
+                CONTENT_LENGTH=str(len(data)),
+                wsgi_input=wsgi_input,
+                **kwargs
+            ),
+            follow,
+            history,
+            check_status,
+        )
+
+    def post_multipart(self, PATH_INFO='/', data=None, files=None,
+                       charset='UTF-8', follow=True, history=True,
+                       check_status=True, **kwargs):
         """
         POST a request to the given URI using multipart/form-data encoding.
 
-        :param PATH_INFO: The path to request from the application. This must
-                          be a URL encoded string.
+        PATH_INFO
+            The path to request from the application. This must be a URL
+            encoded string.
 
-        :param data: POST data to be sent to the application, must be either a
-                     dict or list of ``(name, value)`` tuples.
+        data
+            POST data to be sent to the application, must be either a
+            dict or list of ``(name, value)`` tuples.
 
-        :param charset: Encoding used for any unicode values encountered in
-                        ``data``
+        files
+            list of ``(name, filename, content_type, data)`` tuples. ``data``
+            may be either a byte string, iterator or file-like object.
+
+        history
+            If true, a new entry will be added to the history attribute for the
+            resulting TestAgent object
+
+        check_status
+            If true, any non success HTTP status code will result in an
+            AssertionError being raised
 
-        :param files: list of ``(name, filename, content_type, data)`` tuples.
-                      ``data`` may be either a byte string, iterator or
-                      file-like object.
         """
+
         if data is None:
             data = {}
 
@@ -1245,6 +1244,10 @@ class TestAgent(object):
         if files is None:
             files = []
 
+        if self.request:
+            PATH_INFO = url_join_same_server(self.request.url,
+                                             PATH_INFO)
+
         boundary = b'----------------------------------------BoUnDaRyVaLuE'
 
         def add_headers(key, value):
@@ -1264,8 +1267,10 @@ class TestAgent(object):
             else:
                 if not isinstance(value, bytes):
                     value = value.encode(charset)
-                headers = [('Content-Disposition',
-                            'form-data; name="%s"' % (key,))]
+                headers = [
+                    ('Content-Disposition',
+                    'form-data; name="%s"' % (key,))
+                ]
                 return headers, value
 
         items = chain(
@@ -1297,14 +1302,22 @@ class TestAgent(object):
             post_data.write(b'--' + boundary)
         post_data.write(b'--' + CRLF)
         length = post_data.tell()
-        kwargs.setdefault('method', 'POST')
+        post_data.seek(0)
         kwargs.setdefault('CONTENT_LENGTH', str(length))
-        kwargs.setdefault('CONTENT_TYPE', unicode_to_environ(
-                                            'multipart/form-data; boundary=' +
-                                            boundary.decode('ascii')))
-
-        return self._request(PATH_INFO, post_data.getvalue(),
-                             charset=charset, **kwargs)
+        return self._request(
+            self.make_environ(
+                'POST',
+                PATH_INFO,
+                CONTENT_TYPE=unicode_to_environ(
+                                'multipart/form-data; boundary=' +
+                                boundary.decode('ascii')),
+                wsgi_input=post_data,
+                **kwargs
+            ),
+            follow=follow,
+            history=history,
+            check_status=check_status,
+        )
 
     def reload(self, follow=True, check_status=True):
         """
@@ -1314,19 +1327,17 @@ class TestAgent(object):
         refilled on the reloaded page, provided that the reloaded page has
         exactly the same fields present in the same order.
         """
-        if self.options['logger']:
-            self.options['logger'].debug("Reload: %s",
-                                         self.request.request_path)
+        self.options['logger'] and self.options['logger'].debug("Reload: %s",
+                                          self.request.path)
         env = self.request.environ.copy()
-        del env['pesto.request']
+        del env['fresco.request']
 
         wsgi_input = env['wsgi.input']
         if isinstance(wsgi_input, wsgiref.validate.InputWrapper):
             wsgi_input = wsgi_input.input
         wsgi_input.seek(0)
         env['wsgi.input'] = wsgi_input
-        agent = self._wsgi_request(env, follow=follow,
-                                   check_status=check_status)
+        agent = self._request(env, follow=follow, check_status=check_status)
 
         for src, dst in zip(
             self.find('//input|//textarea|//option'),
@@ -1557,8 +1568,8 @@ class TestAgent(object):
             raise NotARedirectError(
                 "Can't follow non-redirect response (got %s for %s %s)" % (
                     self.response.status,
-                    self.request.request_method,
-                    self.request.request_path
+                    self.request.method,
+                    self.request.path
                 )
             )
 
@@ -1628,13 +1639,12 @@ class TestAgent(object):
         host = self.environ_defaults['SERVER_NAME']
         port = int(self.environ_defaults['SERVER_PORT'])
 
+        server = make_server(host, port, PassStateWSGIApp(self))
         url = self.request.make_uri(netloc='{0}:{1}'.format(host, port))
-        server = make_server(host, port,
-                             PassStateWSGIApp(self, self.request.request_path))
         if open_in_browser:
             webbrowser.open_new_tab(url)
-        print("\nStarting HTTP server on {0}\n"
-              "Press ctrl-c to exit.".format(url))
+        print("\nStarting HTTP server on {0}:{1}. "
+              "Press ctrl-c to exit.".format(host, port))
         try:
             server.serve_forever()
         except KeyboardInterrupt:
@@ -1663,45 +1673,24 @@ class PassStateWSGIApp(object):
     Used by ``TestAgent.serve``.
     """
 
-    def __init__(self, testagent, initial_path):
+    def __init__(self, testagent):
         self.first_request_served = False
         self.testagent = testagent
-        self.initial_path = initial_path
 
     def __call__(self, environ, start_response):
         if self.first_request_served:
             return self.testagent.validated_app(environ, start_response)
-        if Request(environ).request_path != self.initial_path:
-            return self.redirect_to_initial_path(environ, start_response)
+        self.first_request_served = True
         return self.first_request(environ, start_response)
 
-    def redirect_to_initial_path(self, environ, start_response):
-        start_response('302 Found', [('Content-Type', 'text/html'),
-                                     ('Location', self.initial_path)])
-        return ['<a href="{0}">{0}</a>'.format(self.initial_path)
-                                       .encode('utf8')]
-
     def first_request(self, environ, start_response):
-        self.first_request_served = True
         response = self.testagent.response
         mimetype, charset = parse_header(response.get_header('Content-Type'))
         charset = dict(charset).get('charset', 'UTF-8')
-        self.testagent._read_response()
-
         if is_html(response) and self.testagent._lxml is not None:
-            content = tostring(self.testagent.lxml, encoding=charset)
-        else:
-            content = self.testagent._body
-        assert isinstance(content, bytes)
+            response = response.replace(content=[tostring(self.testagent.lxml,
+                                                          encoding=charset)])
 
-        if is_html(response):
-            original_url = base_url(self.testagent.request.environ)
-            served_url = base_url(environ)
-            if original_url != served_url:
-                content = content.replace(original_url.encode(charset),
-                                          served_url.encode(charset))
-
-        response = response.replace(content=[content])
         for key, morsel in self.testagent.cookies.items():
             response = response.add_header('Set-Cookie', morsel.OutputString())
         return response(environ, start_response)
@@ -1729,30 +1718,14 @@ def url_join_same_server(baseurl, url):
         ValueError: URI links to another server: http://example.org/bar
 
     """
-    url = uri_join(baseurl, url)
+    url = url_join(baseurl, url)
     url = urlparse(url)
-    baseurl = urlparse(baseurl)
-    if normalize_host(baseurl.scheme, baseurl.netloc) != \
-            normalize_host(url.scheme, url.netloc):
-        raise ValueError("URI links to another server: %s" %
+    if urlparse(baseurl)[:2] != url[:2]:
+        raise ValueError("URI links to another server: %s" %\
                          (urlunparse(url),))
     return urlunparse(('', '') + url[2:])
 
 
-def normalize_host(scheme, host):
-    """
-    Normalize the host part of the URL
-    """
-    host, _, port = host.partition(':')
-    if port == '80' and scheme in ('http', None):
-        port = ''
-    if port == '443' and scheme == 'https':
-        port = ''
-    if port:
-        return '{0}:{1}'.format(host, port)
-    return host
-
-
 def parse_cookies(response):
     """
     Return a ``Cookie.BaseCookie`` object populated from cookies parsed from
@@ -1801,33 +1774,3 @@ def guess_expression_flavor(expr):
     if '@' in expr:
         return 'xpath'
     return 'css'
-
-
-def unicode_to_environ(s, enc='UTF-8'):
-    """
-    Return a unicode string encoded for a WSGI environ value.
-    This implementation good for python2 only
-    """
-    return s.encode(enc, 'surrogateescape')
-
-
-def base_url(environ):
-    """
-    Return the base URL for the request (ie everything up to SCRIPT_NAME;
-    PATH_INFO and QUERY_STRING are not included)
-    """
-    url = environ['wsgi.url_scheme'] + '://'
-
-    if environ.get('HTTP_HOST'):
-        url += environ['HTTP_HOST']
-    else:
-        url += environ['SERVER_NAME']
-
-        if environ['wsgi.url_scheme'] == 'https':
-            if environ['SERVER_PORT'] != '443':
-                url += ':' + environ['SERVER_PORT']
-        elif environ['SERVER_PORT'] != '80':
-                url += ':' + environ['SERVER_PORT']
-
-    url += quote(environ.get('SCRIPT_NAME', ''))
-    return url
